// game.js for Perlenspiel 2.3

/*
Perlenspiel is a scheme by Professor Moriarty (bmoriarty@wpi.edu).
Perlenspiel is Copyright © 2009-12 Worcester Polytechnic Institute.
This file is part of Perlenspiel.

Perlenspiel is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published
by the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Perlenspiel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You may have received a copy of the GNU Lesser General Public License
along with Perlenspiel. If not, see <http://www.gnu.org/licenses/>.
*/

// The following comment lines are for JSLint. Don't remove them!

/*jslint nomen: true, white: true */
/*global PS */

// This is a template for creating new Perlenspiel games
// All of the functions below MUST exist, or the engine will stop and complain!

// PS.Init ()
// Initializes the game
// This function normally includes a call to PS.GridSize (x, y)
// where x and y are the desired initial dimensions of the grid
// options = a table with optional parameters; see documentation for details

var dynamicInterfaceState = { "Equipment": 0,
    "Inventory": 1,
    "Stats": 2, 
    "Interact": 3, 
    "Magic": 4,
    "Attack": 5
}

var mouseMode = {
    "Normal": 0,
    "Attack": 1,
    "Magic": 2,
    "Interact": 3
}

var interfaceState = dynamicInterfaceState.Stats;
var stringInterfaceState = "Stats";
var heroStatisticsNeedRefreshed = true;
var dynamicUINeedsRefreshed = true;
var worldViewNeedsRefreshed = true;
var aisTurn = false;
var mapInitialized = false;

var currentMouseMode = mouseMode.Normal;
var defaultEmptyItem = new Item("Empty", 0, 0, 0, 0, 0, -1, 0);
var gameOver = false;
var openingInstructionsScreen = true;

var hero;
var theWorld;
var currentWave = 0;

var enemiesInMap = new Array();
var alliesInMap = new Array();

var spellOnMouse = -1;

PS.Init = function (options)
{
	"use strict";

    // change to the grid dimensions you want

	PS.Clock(5);

	PS.GridSize(32, 32);
	openingInstructionsScreen = true;
	InstructionsScreen();
	
    // Put any other init code here
};

function InstructionsScreen() {
    for (var x = 0; x < 32; x++) {
        for (var y = 0; y < 32; y++) {
            PS.BeadColor(x, y, PS.COLOR_BLACK);
            PS.BeadGlyph(x, y, " ");
            PS.BeadGlyphColor(x, y, PS.COLOR_WHITE);
        }
    }

    var stringInstructionScreen = new Array();

    stringInstructionScreen.push("This is the story about a");
    stringInstructionScreen.push("heroic knight who defended");
    stringInstructionScreen.push("King Ramsus until the very");
    stringInstructionScreen.push("end.  All of the other ");
    stringInstructionScreen.push("knights had abandoned their");
    stringInstructionScreen.push("King, but one knight stood");
    stringInstructionScreen.push("firm against insurmountable");
    stringInstructionScreen.push("odds.  This is his tale...");
    stringInstructionScreen.push(" ");
    stringInstructionScreen.push(" ");
    stringInstructionScreen.push("Instructions: ");
    stringInstructionScreen.push("A wave of enemy bandits will");
    stringInstructionScreen.push("come from one of the four ");
    stringInstructionScreen.push("bridges.  They will attempt to");
    stringInstructionScreen.push("kill you, your King, the Wizard");
    stringInstructionScreen.push("the Merchant, and the ");
    stringInstructionScreen.push("Blacksmith.  You can spend your");
    stringInstructionScreen.push("gold learning spells from the ");
    stringInstructionScreen.push("Wizard, or on upgrades from the");
    stringInstructionScreen.push("Blacksmith.  The Blacksmith ");
    stringInstructionScreen.push("will upgrade your existing gear");
    stringInstructionScreen.push("at a steadily increasing price.");
    stringInstructionScreen.push(" ");
    stringInstructionScreen.push("The Merchant will buy old gear ");
    stringInstructionScreen.push("from you.  Guard the King.");
    stringInstructionScreen.push("While it isn't good to let");
    stringInstructionScreen.push("any of your allies die, if the");
    stringInstructionScreen.push("King dies, you will fail.");
    stringInstructionScreen.push(" ");
    stringInstructionScreen.push("Use the arrow keys to move.");
    stringInstructionScreen.push("Press any key to start the game!");

    for (var y = 0; y < stringInstructionScreen.length; y++) {
        for (var x = 0; x < stringInstructionScreen[y].length; x++) {
            PS.BeadGlyph(x, y, stringInstructionScreen[y][x]);
        }
    }
}

function BeginTheGame() {

    for (var x = 0; x < 32; x++) {
        for (var y = 0; y < 32; y++) {
            PS.BeadFlash(x, y, false);
            PS.BeadGlyph(x, y, " ");
        }
    }

    hero = new Hero(5, 5, 5, 5, 50, "☺");
    currentWave = 0;
    alliesInMap = new Array();
    enemiesInMap = new Array();
    theWorld = new World();
    DrawBasicInterface();
    hero.InitializeStats();
    hero.CalculateStats();
    DrawHMGPoints();
    theWorld.Setup();

    PS.BeadGlyphColor(16, 11, PS.COLOR_BLACK);
    PS.BeadGlyph(16, 11, "☺");

    worldViewNeedsRefreshed = true;

    for (var x = 0; x < 32; x++) {
        for (var y = 0; y < 23; y++) {
            PS.BeadGlyphColor(x, y, PS.COLOR_BLACK);
        }
    }
}

function BeginWave() {

    if (currentWave == 0) {
        alliesInMap.push(new ArtificialLifeform("King", true, 1, 1, 1, 0, "♕", 1, 0, 31, 25));
        alliesInMap.push(new ArtificialLifeform("Blacksmith", true, 15, 5, 10, 0, "♖", 20, 0, 11, 31));
        alliesInMap.push(new ArtificialLifeform("Wizard", true, 5, 15, 5, 50, "♫", 20, 0, 51, 31));
        alliesInMap.push(new ArtificialLifeform("Merchant", true, 15, 5, 10, 0, "$", 20, 0, 31, 11));

        for (var i = 0 ; i < alliesInMap.length; i++) {
            alliesInMap[i].Initialize();
        }
    }

    var waveStrength = 10 * currentWave;
    var waveArmor = 10 * currentWave;
    var waveHP = 25 * (Math.pow(2, currentWave));

    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 3, 30));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 3, 31));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 3, 32));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 2, 30));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 2, 31));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 2, 32));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 1, 30));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 1, 31));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 1, 32));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 60, 30));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 60, 31));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 60, 32));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 61, 30));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 61, 31));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 61, 32));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 62, 30));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 62, 31));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 62, 32));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 30, 3));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 31, 3));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 32, 3));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 30, 2));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 31, 2));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 32, 2));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 30, 1));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 31, 1));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 32, 1));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 30, 60));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 31, 60));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 32, 60));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 30, 61));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 31, 61));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 32, 61));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 30, 62));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 31, 62));
    enemiesInMap.push(new ArtificialLifeform("Bandit", false, waveStrength, 15, waveArmor, 5, "☠", waveHP, 10, 32, 62));

    if (hero.currentMana + (hero.maxMana * .1) > hero.maxMana) {
        hero.currentMana = hero.maxMana;
    }
    else {
        hero.currentMana += hero.maxMana * .1;
    }

    if (hero.currentHP + (hero.maxHP * .10) > hero.maxHP) {
        hero.currentHP = hero.maxHP;
    }
    else {
        hero.currentHP += hero.maxHP * .1;
    }

    for (var i = 0; i < enemiesInMap.length; i++) {
        enemiesInMap[i].Initialize();
    }

    currentWave++;

    dynamicUINeedsRefreshed = true;
}
// PS.Click (x, y, data)
// This function is called whenever a bead is clicked
// It doesn't have to do anything
// x = the x-position of the bead on the grid
// y = the y-position of the bead on the grid
// data = the data value associated with this bead, 0 if none has been set
// options = a table with optional parameters; see documentation for details

PS.Click = function (x, y, data, options)
{
	"use strict";
		
	if (!gameOver && !aisTurn) {
	    if (PS.BeadColor(x, y) == PS.COLOR_RED) {
	        hero.potentialTarget = 0;

	        if (interfaceState != PS.BeadData(x, y)) {
	            interfaceState = PS.BeadData(x, y);
	            dynamicUINeedsRefreshed = true;
	        }
	    }
	    else if (PS.BeadColor(x, y) == PS.COLOR_YELLOW) {
	        hero.EquipItem(y - 24);
	    }
	    else if (y < 23 && PS.BeadData(x, y).occupiedBy != 0) {
	        if (currentMouseMode == mouseMode.Attack) {
	            if (!PS.BeadData(x, y).occupiedBy.friendly) {
	                if (Math.abs(hero.worldPositionX - PS.BeadData(x, y).occupiedBy.worldPosition_X) <= 1 &&
                        Math.abs(hero.worldPositionY - PS.BeadData(x, y).occupiedBy.worldPosition_Y) <= 1) {
	                    Attack(hero, PS.BeadData(x, y).occupiedBy);
	                    aisTurn = true;
	                }
	                else {
	                    PS.StatusText("Get closer to attack!");
	                }
	            }
	        }
	        else if (currentMouseMode == mouseMode.Magic && spellOnMouse > -1) {
	            if (CastSpell(hero, PS.BeadData(x, y).occupiedBy, spellOnMouse)) {
	                aisTurn = true;
	                dynamicUINeedsRefreshed = true;
	                spellOnMouse = -1;
	            }
	        }
	        else if (currentMouseMode == mouseMode.Interact) {
	            PerformInteract(PS.BeadData(x,y).occupiedBy.name);
	        }
	    }
	    else if (PS.BeadColor(x, y) == PS.COLOR_MAGENTA) {
	        spellOnMouse = y - 24;

	        if (spellOnMouse == 6 || spellOnMouse == 7) {
	            if (CastSpell(hero, true, spellOnMouse)) {
	                spellOnMouse = -1;
	                aisTurn = true;
	                dynamicUINeedsRefreshed = true;
	                spellOnMouse = -1;
	            }
	        }
	    }
	}
};

// PS.Release (x, y, data)
// This function is called whenever a mouse button is released over a bead
// It doesn't have to do anything
// x = the x-position of the bead on the grid
// y = the y-position of the bead on the grid
// data = the data value associated with this bead, 0 if none has been set
// options = a table with optional parameters; see documentation for details

PS.Release = function (x, y, data, options)
{
	"use strict";

	// Put code here for when the mouse button is released over a bead	
};

// PS.Enter (x, y, button, data)
// This function is called whenever the mouse moves over a bead
// It doesn't have to do anything
// x = the x-position of the bead on the grid
// y = the y-position of the bead on the grid
// data = the data value associated with this bead, 0 if none has been set
// options = a table with optional parameters; see documentation for details

PS.Enter = function (x, y, data, options)
{
    "use strict";

    if (!gameOver) {
        if (currentMouseMode == mouseMode.Attack || currentMouseMode == mouseMode.Interact) {
            if (y < 23) {
                if (PS.BeadData(x, y).occupiedBy != 0) {
                    dynamicUINeedsRefreshed = true;
                    hero.potentialTarget = PS.BeadData(x, y).occupiedBy;
                }
            }
        }
    }

	// Put code here for when the mouse enters a bead	
};

// PS.Leave (x, y, data)
// This function is called whenever the mouse moves away from a bead
// It doesn't have to do anything
// x = the x-position of the bead on the grid
// y = the y-position of the bead on the grid
// data = the data value associated with this bead, 0 if none has been set
// options = a table with optional parameters; see documentation for details

PS.Leave = function (x, y, data, options)
{
	"use strict";
	
	// Put code here for when the mouse leaves a bead	
};

// PS.KeyDown (key, shift, ctrl)
// This function is called whenever a key on the keyboard is pressed
// It doesn't have to do anything
// key = the ASCII code of the pressed key, or one of the following constants:
// Arrow keys = PS.ARROW_UP, PS.ARROW_DOWN, PS.ARROW_LEFT, PS.ARROW_RIGHT
// Function keys = PS.F1 through PS.F1
// shift = true if shift key is held down, false otherwise
// ctrl = true if control key is held down, false otherwise
// options = a table with optional parameters; see documentation for details

PS.KeyDown = function (key, shift, ctrl, options)
{
	"use strict";

	// Put code here for when a key is pressed	
};

// PS.KeyUp (key, shift, ctrl)
// This function is called whenever a key on the keyboard is released
// It doesn't have to do anything
// key = the ASCII code of the pressed key, or one of the following constants:
// Arrow keys = PS.ARROW_UP, PS.ARROW_DOWN, PS.ARROW_LEFT, PS.ARROW_RIGHT
// Function keys = PS.F1 through PS.F12
// shift = true if shift key is held down, false otherwise
// ctrl = true if control key is held down, false otherwise
// options = a table with optional parameters; see documentation for details

PS.KeyUp = function (key, shift, ctrl, options)
{
    "use strict";

    // I had to do this because there was a way to force yourself into walls and cause the terrain to be
    // perpetually flagged as occupied

    if (openingInstructionsScreen && key != 82) {
        openingInstructionsScreen = false;
        BeginTheGame();
    }
    else if (!gameOver) {
        if (key != 82) {
            if (worldViewNeedsRefreshed == false && aisTurn != true) {
                if (key == PS.ARROW_RIGHT) {
                    if (hero.worldPositionX < 64) {
                        hero.NewPlayerPosition(1); // Try to move right
                    }
                }

                if (key == PS.ARROW_LEFT) {
                    if (hero.worldPositionX > 0) {
                        hero.NewPlayerPosition(3); // Try to move left
                    }
                }

                if (key == PS.ARROW_UP) {
                    if (hero.worldPositionY > 0) {
                        hero.NewPlayerPosition(0); // Try to move up
                    }
                }

                if (key == PS.ARROW_DOWN) {
                    if (hero.worldPositionY < 64) {
                        hero.NewPlayerPosition(2); // Try to move down
                    }
                }
            }
        }
        else {
            PS.Init();
        }
    }
};

// PS.Wheel (dir)
// This function is called whenever the mouse wheel moves forward or backward
// It doesn't have to do anything
// dir = PS.FORWARD if mouse wheel moves forward, PS.BACKWARD if backward
// options = a table with optional parameters; see documentation for details

PS.Wheel = function (dir, options)
{
	"use strict";

	// Put code here for when mouse wheel is moved
};

// PS.Tick ()
// This function is called on every clock tick
// if a timer has been activated with a call to PS.Timer()
// It doesn't have to do anything
// options = a table with optional parameters; see documentation for details

PS.Tick = function (options)
{
    "use strict";

    if (!openingInstructionsScreen) {
        if (hero.currentHP <= 0 && !gameOver) { // The GameOver screen only needs to be drawn once
            GameOver();
        }
        else if (!gameOver) {

            if (dynamicUINeedsRefreshed == true) {
                DrawDynamicInterfaceElements();
                DrawHMGPoints();
            }

            if (worldViewNeedsRefreshed == true) {
                theWorld.Draw(hero.worldPositionX, hero.worldPositionY);
            }

            if (aisTurn) {
                for (var i = 0; i < enemiesInMap.length; i++) {
                    if (enemiesInMap[i].currentHP > 0) {
                        enemiesInMap[i].ProcessThoughtPattern();
                    }
                    else {
                        theWorld.worldTerrain[enemiesInMap[i].worldPosition_X][enemiesInMap[i].worldPosition_Y].occupiedBy = 0;
                        hero.Gold += enemiesInMap[i].goldValue;
                        
                        if (enemiesInMap[i].Loot.eqSlot != -1) {
                            hero.AcquiredItem(enemiesInMap[i].Loot);
                        }

                        enemiesInMap.splice(i, 1);
                        i--;
                        DrawHMGPoints();
                    }
                }

                for (var i = 0; i < alliesInMap.length; i++) {
                    if (alliesInMap[i].currentHP > 0) {
                        alliesInMap[i].ProcessThoughtPattern();
                    }
                    else {
                        theWorld.worldTerrain[alliesInMap[i].worldPosition_X][alliesInMap[i].worldPosition_Y].occupiedBy = 0;
                        alliesInMap.splice(i, 1);
                        i--;
                    }
                }

                aisTurn = false;
                worldViewNeedsRefreshed = true;
            }
        }

        if (enemiesInMap.length == 0) {
            BeginWave();
        }
    }
	// Put code here to handle clock ticks
};

function DrawBasicInterface()
{
    var rows, columns;
    var i = 0;

    for (rows = 31; rows > 22; rows--)
    {
        for (columns = 0; columns < 14; columns++)
        {
            if (columns < 14) {
                PS.BeadColor(columns, rows, PS.COLOR_BLACK);
            }
        }
    }

    var stringEquipment = "Equipment";
    var stringInventory = "Inventory";
    var stringStats = "Stats";
    var stringInteract = "Interact";
    var stringMagic = "Magic";
    var stringAttack = "Attack";

    // This next section may seem painfully drawn out, but it is done like this so that each section flows in the code.
    columns = 31;
    PS.BeadColor(1, columns, PS.COLOR_RED);
    PS.BeadData(1, columns, dynamicInterfaceState.Equipment);

    for (i = 0; i < stringEquipment.length && 2 + 1 < 32; i++) {
        PS.BeadGlyph(2 + i, columns, stringEquipment[i]);
    }

    columns--;
    PS.BeadColor(1, columns, PS.COLOR_RED);
    PS.BeadData(1, columns, dynamicInterfaceState.Inventory);

    for (i = 0; i < stringInventory.length && 2 + 1 < 32; i++) {
        PS.BeadGlyph(2 + i, columns, stringInventory[i]);
    }

    columns--;
    PS.BeadColor(1, columns, PS.COLOR_RED);
    PS.BeadData(1, columns, dynamicInterfaceState.Stats);

    for (i = 0; i < stringStats.length && 2 + 1 < 32; i++) {
        PS.BeadGlyph(2 + i, columns, stringStats[i]);
    }

    columns--;
    PS.BeadColor(1, columns, PS.COLOR_RED);
    PS.BeadData(1, columns, dynamicInterfaceState.Interact);

    for (i = 0; i < stringInteract.length && 2 + 1 < 32; i++) {
        PS.BeadGlyph(2 + i, columns, stringInteract[i]);
    }

    columns--;
    PS.BeadColor(1, columns, PS.COLOR_RED);
    PS.BeadData(1, columns, dynamicInterfaceState.Magic);

    for (i = 0; i < stringMagic.length && 2 + 1 < 32; i++) {
        PS.BeadGlyph(2 + i, columns, stringMagic[i]);
    }

    columns--;
    PS.BeadColor(1, columns, PS.COLOR_RED);
    PS.BeadData(1, columns, dynamicInterfaceState.Attack);

    for (i = 0; i < stringAttack.length && 2 + 1 < 32; i++) {
        PS.BeadGlyph(2 + i, columns, stringAttack[i]);
    }
}

function DrawDynamicInterfaceElements() {

    var columns = 23;
    var dynamicUIStart = 14;

    var x, y;

    for (x = 14; x < 32; x++) {
        for (y = 31; y > 22; y--) {
            PS.BeadColor(x, y, [64, 64, 64]);
            PS.BeadGlyph(x, y, " ");
        }
    }

    if (interfaceState == dynamicInterfaceState.Equipment) {
        stringInterfaceState = "Equipment";
        mouseMode.Normal;
        DrawEquipment(dynamicUIStart, columns + 1);
    }
    else if (interfaceState == dynamicInterfaceState.Inventory) {
        stringInterfaceState = "Inventory";
        mouseMode.Normal;
        DrawInventory(dynamicUIStart, columns + 1);
    }
    else if (interfaceState == dynamicInterfaceState.Stats) {
        stringInterfaceState = "Stats";
        mouseMode.Normal;
        DrawStats(dynamicUIStart, columns + 1);
    }
    else if (interfaceState == dynamicInterfaceState.Interact) {
        stringInterfaceState = "Interact";
        currentMouseMode = mouseMode.Interact;
        DrawInteract(dynamicUIStart, columns + 1);
    }
    else if (interfaceState == dynamicInterfaceState.Magic) {
        stringInterfaceState = "Magic";
        currentMouseMode = mouseMode.Magic;
        DrawMagic(dynamicUIStart, columns + 1);
    }
    else if (interfaceState == dynamicInterfaceState.Attack) {
        stringInterfaceState = "Attack";
        currentMouseMode = mouseMode.Attack;
        DrawAttack(dynamicUIStart, columns + 1);
    }

    for (i = 0; i < stringInterfaceState.length; i++) {
        PS.BeadGlyph(14 + i, columns, stringInterfaceState[i]);
    }

    dynamicUINeedsRefreshed = false;
}

function DrawHMGPoints() {
    // Draw HP, MP, GP
    var columns = 25;
    var i;
    var stringHP = "HP:" + hero.currentHP + "/" + hero.maxHP;
    var stringMana = "MP:" + hero.currentMana + "/" + hero.maxMana;
    var stringGold = "GP:" + hero.Gold;

    for (i = 0; i < 12; i++) {
        for (var y = 23; y < 26; y++) {
            PS.BeadGlyph(2 + i, y, " ");
        }
    }

    for (i = 0; i < stringGold.length && 2 + 1 < 32; i++) {
        PS.BeadGlyphColor(2 + i, columns, PS.COLOR_YELLOW);
        PS.BeadGlyph(2 + i, columns, stringGold[i]);
    }

    columns--;
    for (i = 0; i < stringMana.length && 2 + 1 < 32; i++) {
        PS.BeadGlyphColor(2 + i, columns, [0, 255, 255]);
        PS.BeadGlyph(2 + i, columns, stringMana[i]);
    }

    columns--;
    for (i = 0; i < stringHP.length && 2 + 1 < 32; i++) {
        PS.BeadGlyphColor(2 + i, columns, PS.COLOR_RED);
        PS.BeadGlyph(2 + i, columns, stringHP[i]);
    }
}

function DrawStats(x, y) {
    var stringStrength = "Strength: " + hero.Strength;
    var stringIntelligence = "Intelligence: " + hero.Intelligence;
    var stringConstitution = "Constitution: " + hero.Constitution;
    var stringArmor = "Armor: " + hero.Armor;
    var stringCritHitChance = "Crit: " + hero.CriticalHitChance / 10 + "%";

    for (i = 0; i < stringStrength.length && x+i < 32; i++) {
        PS.BeadGlyph(x + i, y, stringStrength[i]);
    }

    y++;
    for (i = 0; i < stringIntelligence.length && x+i < 32; i++) {
        PS.BeadGlyph(x + i, y, stringIntelligence[i]);
    }

    y++;
    for (i = 0; i < stringConstitution.length && x+i < 32; i++) {
        PS.BeadGlyph(x + i, y, stringConstitution[i]);
    }

    y++;
    for (i = 0; i < stringArmor.length && x+i < 32; i++) {
        PS.BeadGlyph(x + i, y, stringArmor[i]);
    }

    y++;
    for (i = 0; i < stringCritHitChance.length && x+i < 32; i++) {
        PS.BeadGlyph(x + i, y, stringCritHitChance[i]);
    }
}

function DrawInteract(x, y) {
    if (hero.potentialTarget != 0) {
        var potentialTarget = hero.potentialTarget;
        var stringMessage = new Array();

        if (hero.potentialTarget.name == "King") {
            stringMessage.push("This is your King!");
            stringMessage.push("It is your duty to");
            stringMessage.push("protect him to the");
            stringMessage.push("bitter end!");
        }
        else if (hero.potentialTarget.name == "Blacksmith") {
            stringMessage.push("This is the       ");
            stringMessage.push("Blacksmith!  He   ");
            stringMessage.push("can upgrade your  ");
            stringMessage.push("equipment for the ");
            stringMessage.push("low price of only ");
            stringMessage.push(CalculateHeroUpgradeValue() + " gold!");
            stringMessage.push("Click him to      ");
            stringMessage.push("upgrade your gear!");
        }
        else if (hero.potentialTarget.name == "Wizard") {
            var spellOffered = -1;
            for (var i = 0; i < hero.spellsLearned.length && spellOffered == -1; i++) {
                if(hero.spellsLearned[i] == false) {
                    stringMessage.push("I am the Wizard!");
                    stringMessage.push("For " + CalculateNextSpellCost() + " gold,");
                    stringMessage.push("I can teach you");
                    stringMessage.push("a new spell!");
                    spellOffered = i;
                }
            }

            if(spellOffered == -1) {
                stringMessage.push("I'm sorry, you ");
                stringMessage.push("know all of the");
                stringMessage.push("spells I can ");
                stringMessage.push("teach you.");
            }
        }
        else if (hero.potentialTarget.name == "Merchant") {
            stringMessage.push("I am the merchant!");
            stringMessage.push("I will buy every ");
            stringMessage.push("single item in ");
            stringMessage.push("your inventory!");
            stringMessage.push("No qualms!  No ");
            stringMessage.push("questions!  Just ");
            stringMessage.push("click me!");
        }
        else if (hero.potentialTarget.name == "Bandit") {
            stringMessage.push("This is a bandit!");
            stringMessage.push("He is searching ");
            stringMessage.push("the city to find ");
            stringMessage.push("people loyal to ");
            stringMessage.push("King Ramsus in ");
            stringMessage.push("order to kill them");
        }
        else if (hero.potentialTarget.name == "You") {
            stringMessage.push("This is you! Yay!");
        }

        for (var row = 0; row < stringMessage.length; row++) {
            for (var i = 0; i < stringMessage[row].length && (x + i) < 32; i++) {
                PS.BeadGlyph(x + i, y + row, stringMessage[row][i]);
            }
        }
    }
}

function CalculateHeroUpgradeValue() {
    var totalFees = Math.Pow(((hero.blacksmithUpgrades + 1) * 25), 1.25);

    return totalFees;
}

function CalculateNextSpellCost() {
    var unlearned = true;

    for (var i = 0; i < hero.spellsLearned.length && unlearned == true; i++) {
        if (hero.spellsLearned[i] == false) {
            unlearned = false;
            return 100 * Math.pow(2, i);
        }
    }
}

function PerformInteract(nameOfNPC) {

    if (nameOfNPC == "Blacksmith") {
        if (hero.Gold >= CalculateHeroUpgradeValue()) {
            PS.StatusText("I upgraded your items!");
            hero.Gold -= CalculateHeroUpgradeValue();
            hero.UpgradeItems();
        }
        else {
            PS.StatusText("You need more money to upgrade!");
        }
    }
    else if (nameOfNPC == "Wizard") {
        if (hero.Gold >= CalculateNextSpellCost()) {
            var unlearned = true;

            for (var i = 0; i < hero.spellsLearned.length && unlearned == true; i++) {
                if (hero.spellsLearned[i] == false) {
                    hero.Gold -= CalculateNextSpellCost();
                    hero.spellsLearned[i] = true;
                    unlearned = false;
                }
            }
        }
        else {
            PS.StatusText("You need money to learn!");
        }
    }
    else if (nameOfNPC == "Merchant") {
        for (var i = 0 ; i < hero.inventory.length; i++) {
            if (hero.inventory[i].eqSlot != -1) {
                hero.Gold += hero.inventory[i].value;
                hero.inventory[i] = defaultEmptyItem;
            }
        }
    }

    dynamicUINeedsRefreshed = true;
}

function DrawMagic(x, y) {
    var stringUnknown = "????";
    var stringHeal = "Heal";
    var stringLightningStrike = "Lightning Strike";
    var stringSuperHeal = "Super Heal";
    var stringFireBlast = "Fire Blast";
    var stringFullHeal = "Full Heal";
    var stringDeathTouch = "Death Touch";
    var stringWorldHeal = "World Heal";
    var stringInferno = "Inferno";

    if (hero.spellsLearned[0] == true) {
        for (var i = 0; i <= stringHeal.length && (x + i < 32) ; i++) {
            if (i == 0) {
                if (hero.currentMana >= 10) {
                    PS.BeadColor(x + i, y, PS.COLOR_MAGENTA);
                }
                else {
                    PS.BeadColor(x + i, y, [64, 64, 64]);
                }
            }
            else {
                PS.BeadGlyph(x + i, y, stringHeal[i - 1]);
            }
        }
    }
    else {
        for (var i = 0; i <= stringUnknown.length && (x + i < 32) ; i++) {
            if (i == 0) {
                PS.BeadColor(x + i, y, [64, 64, 64]);
            }
            else {
                PS.BeadGlyph(x + i, y, stringUnknown[i - 1]);
            }
        }
    }

    y++;
    if (hero.spellsLearned[1] == true) {
        for (var i = 0; i <= stringLightningStrike.length && (x + i < 32) ; i++) {
            if (i == 0) {
                if (hero.currentMana >= 20) {
                    PS.BeadColor(x + i, y, PS.COLOR_MAGENTA);
                }
                else {
                    PS.BeadColor(x + i, y, [64, 64, 64]);
                }
            }
            else {
                PS.BeadGlyph(x + i, y, stringLightningStrike[i - 1]);
            }
        }
    }
    else {
        for (var i = 0; i <= stringUnknown.length && (x + i < 32) ; i++) {
            if (i == 0) {
                PS.BeadColor(x + i, y, [64, 64, 64]);
            }
            else {
                PS.BeadGlyph(x + i, y, stringUnknown[i - 1]);
            }
        }
    }

    y++;
    if (hero.spellsLearned[2] == true) {
        for (var i = 0; i <= stringSuperHeal.length && (x + i < 32) ; i++) {
            if (i == 0) {
                if (hero.currentMana >= 35) {
                    PS.BeadColor(x + i, y, PS.COLOR_MAGENTA);
                }
                else {
                    PS.BeadColor(x + i, y, [64, 64, 64]);
                }
            }
            else {
                PS.BeadGlyph(x + i, y, stringSuperHeal[i - 1]);
            }
        }
    }
    else {
        for (var i = 0; i <= stringUnknown.length && (x + i < 32) ; i++) {
            if (i == 0) {
                PS.BeadColor(x + i, y, [64, 64, 64]);
            }
            else {
                PS.BeadGlyph(x + i, y, stringUnknown[i - 1]);
            }
        }
    }

    y++;
    if (hero.spellsLearned[3] == true) {
        for (var i = 0; i <= stringFireBlast.length && (x + i < 32) ; i++) {
            if (i == 0) {
                if (hero.currentMana >= 50) {
                    PS.BeadColor(x + i, y, PS.COLOR_MAGENTA);
                }
                else {
                    PS.BeadColor(x + i, y, [64, 64, 64]);
                }
            }
            else {
                PS.BeadGlyph(x + i, y, stringFireBlast[i - 1]);
            }
        }
    }
    else {
        for (var i = 0; i <= stringUnknown.length && (x + i < 32) ; i++) {
            if (i == 0) {
                PS.BeadColor(x + i, y, [64, 64, 64]);
            }
            else {
                PS.BeadGlyph(x + i, y, stringUnknown[i - 1]);
            }
        }
    }

    y++;
    if (hero.spellsLearned[4] == true) {
        for (var i = 0; i <= stringFullHeal.length && (x + i < 32) ; i++) {
            if (i == 0) {
                if (hero.currentMana >= 75) {
                    PS.BeadColor(x + i, y, PS.COLOR_MAGENTA);
                }
                else {
                    PS.BeadColor(x + i, y, [64, 64, 64]);
                }
            }
            else {
                PS.BeadGlyph(x + i, y, stringFullHeal[i - 1]);
            }
        }
    }
    else {
        for (var i = 0; i <= stringUnknown.length && (x + i < 32) ; i++) {
            if (i == 0) {
                PS.BeadColor(x + i, y, [64, 64, 64]);
            }
            else {
                PS.BeadGlyph(x + i, y, stringUnknown[i - 1]);
            }
        }
    }

    y++;
    if (hero.spellsLearned[5] == true) {
        for (var i = 0; i <= stringDeathTouch.length && (x + i < 32) ; i++) {
            if (i == 0) {
                if (hero.currentMana >= 100) {
                    PS.BeadColor(x + i, y, PS.COLOR_MAGENTA);
                }
                else {
                    PS.BeadColor(x + i, y, [64, 64, 64]);
                }
            }
            else {
                PS.BeadGlyph(x + i, y, stringDeathTouch[i - 1]);
            }
        }
    }
    else {
        for (var i = 0; i <= stringUnknown.length && (x + i < 32) ; i++) {
            if (i == 0) {
                PS.BeadColor(x + i, y, [64, 64, 64]);
            }
            else {
                PS.BeadGlyph(x + i, y, stringUnknown[i - 1]);
            }
        }
    }

    y++;
    if (hero.spellsLearned[6] == true) {
        for (var i = 0; i <= stringWorldHeal.length && (x + i < 32) ; i++) {
            if (i == 0) {
                if (hero.currentMana >= 150) {
                    PS.BeadColor(x + i, y, PS.COLOR_MAGENTA);
                }
                else {
                    PS.BeadColor(x + i, y, [64, 64, 64]);
                }
            }
            else {
                PS.BeadGlyph(x + i, y, stringWorldHeal[i - 1]);
            }
        }
    }
    else {
        for (var i = 0; i <= stringUnknown.length && (x + i < 32) ; i++) {
            if (i == 0) {
                PS.BeadColor(x + i, y, [64, 64, 64]);
            }
            else {
                PS.BeadGlyph(x + i, y, stringUnknown[i - 1]);
            }
        }
    }

    y++;
    if (hero.spellsLearned[7] == true) {
        for (var i = 0; i <= stringInferno.length && (x + i < 32) ; i++) {
            if (i == 0) {
                if (hero.currentMana >= 250) {
                    PS.BeadColor(x + i, y, PS.COLOR_MAGENTA);
                }
                else {
                    PS.BeadColor(x + i, y, [64, 64, 64]);
                }
            }
            else {
                PS.BeadGlyph(x + i, y, stringInferno[i - 1]);
            }
        }
    }
    else {
        for (var i = 0; i <= stringUnknown.length && (x + i < 32) ; i++) {
            if (i == 0) {
                PS.BeadColor(x + i, y, [64, 64, 64]);
            }
            else {
                PS.BeadGlyph(x + i, y, stringUnknown[i - 1]);
            }
        }
    }
}

function DrawAttack(x, y) {
    if (hero.potentialTarget != 0) {
        var potentialTarget = hero.potentialTarget;
        var stringTargetName = "Name: " + potentialTarget.name;
        var stringTargetHostile = "Hostile: " + (potentialTarget.friendly == true ? "No" : "Yes");
        var stringCondition = "HP: " + CalculateHealthString(potentialTarget) + "%";
        var stringChanceToHit = "Hit %: " + CalculateChanceToHit(hero, potentialTarget);

        for (i = 0; i < stringTargetName.length && (x + i) < 32; i++) {
            PS.BeadGlyph(x + i, y, stringTargetName[i]);
        }

        y++;
        for (i = 0; i < stringTargetHostile.length && (x + i) < 32; i++) {
            PS.BeadGlyph(x + i, y, stringTargetHostile[i]);
        }

        y++;
        for (i = 0; i < stringCondition.length && (x + i) < 32; i++) {
            PS.BeadGlyph(x + i, y, stringCondition[i]);
        }

        y++;
        for (i = 0; i < stringChanceToHit.length && (x + i) < 32; i++) {
            PS.BeadGlyph(x + i, y, stringChanceToHit[i]);
        }
    }
}

function CalculateChanceToHit(aggressor, target) {
    var rollNeeded;
    rollNeeded = target.Armor - (aggressor.Strength / 2);
    if (rollNeeded > 20) {
        return 0;
    }
    else if (rollNeeded < 1) {
        return 100;
    }
    else {
        return ((20 - rollNeeded) * 5);
    }
}

function CalculateHealthString(target) {
    var percOfHealth = target.currentHP / target.maxHP;

    return percOfHealth * 100;
}

function Attack(aggressor, target) {
    var roll = PS.Random(20);
    var damage = PS.Random((aggressor.Strength + aggressor.Intelligence) / 2);
    roll += (aggressor.Strength / 2);
    if (roll >= target.Armor) {
        if (PS.Random(1000) <= (aggressor.CriticalHitChance)) {
            damage *= 2;
        }

        Damage(target, damage);
    }
    else if (!target.friendly) {
        PS.StatusText("Missed!");
    }
}

function CastSpell(aggressor, target, spellID) {
    //var stringHeal = "Heal";
    //var stringLightningStrike = "Lightning Strike";
    //var stringSuperHeal = "Super Heal";
    //var stringFireBlast = "Fire Blast";
    //var stringFullHeal = "Full Heal";
    //var stringDeathTouch = "Death Touch";
    //var stringWorldHeal = "World Heal";
    //var stringInferno = "Inferno";

    // Spell 6 and spell 7 are map wide spells, so I now iterate through the array of all of them


    if (target == true && (spellID == 6 || spellID == 7)) {
        if (spellID == 6) { // World Heal brings all allies (including the King and the Hero) up to max
            // Also, notice that the negative damage would equate to healing
            for (var i = 0; i < alliesInMap.length; i++) {
                Damage(alliesInMap[i], -enemiesInMap[i].maxHP);
            }

            Damage(hero, -hero.maxHP);

            aggressor.currentMana -= 150;
            DrawHMGPoints();
            return true;
        }
        else if (spellID == 7) { // Inferno reduces every enemies HP to 1
            for (var i = 0; i < enemiesInMap.length; i++) {
                Damage(enemiesInMap[i], enemiesInMap[i].currentHP - 1);
            }

            aggressor.currentMana -= 250;
            DrawHMGPoints();
            return true;
        }
    }

    switch (spellID) {
        case 0:
            if (target.friendly) {
                Damage(target, -25);

                aggressor.currentMana -= 10;

                DrawHMGPoints();

                return true;
            }

            return false;
            break;
        case 1:
            if (!target.friendly) {
                Damage(target, 25);

                aggressor.currentMana -= 20;

                DrawHMGPoints();

                return true;
            }
            return false;

            break;
        case 2:
            if (target.friendly) {
                Damage(target, -75);

                aggressor.currentMana -= 35;

                DrawHMGPoints();

                return true;
            }
            return false;
            break;
        case 3:
            if (!target.friendly) {
                Damage(target, 75);

                aggressor.currentMana -= 50;

                DrawHMGPoints();

                return true;
            }

            return false;
            break;
        case 4:
            if (target.friendly) {
                Damage(target, -target.maxHP);

                aggressor.currentMana -= 75;

                DrawHMGPoints();

                return true;
            }

            return false;

            break;
        case 5:
            if (!target.friendly) {
                Damage(target, target.maxHP);

                aggressor.currentMana -= 100;

                DrawHMGPoints();

                return true;
            }
            return false;
            break;
    }
}

function Damage(target, dmgAmount) {
    if (dmgAmount > target.currentHP) {
        target.currentHP = 0;
    }
    else {
        target.currentHP -= dmgAmount;
    }

    if (target.currentHP > target.maxHP) {
        target.currentHP = target.maxHP;
    }


    if (!target.friendly) {
        if (dmgAmount > 0) {
            PS.StatusText(target.name + " takes " + dmgAmount + " damage!");
        }
        else {
            PS.StatusText(target.name + " is healed " + dmgAmount + " HP!");
        }
    }

    DrawHMGPoints();
    dynamicUINeedsRefreshed = true;
}

function GameOver() {
    gameOver = true;

    for (var x = 0; x < 32; x++) {
        for (var y = 0; y < 32; y++) {
            PS.BeadColor(x, y, PS.COLOR_BLACK);
            PS.BeadGlyph(x, y, " ");
            PS.BeadGlyphColor(x, y, PS.COLOR_WHITE);
        }
    }

    var stringGameOverScreenLines = new Array();

    stringGameOverScreenLines.push("Thus ends our tale...");
    stringGameOverScreenLines.push(" ");
    stringGameOverScreenLines.push("The King's final guard fell ");
    stringGameOverScreenLines.push("to the rebels, and the King ");
    stringGameOverScreenLines.push("fell to them shortly after. ");
    stringGameOverScreenLines.push("King Ramsus would be remembered");
    stringGameOverScreenLines.push("as a tyrant in the annals of");
    stringGameOverScreenLines.push("history, although his elite ");
    stringGameOverScreenLines.push("guard's name would be forgotten");
    stringGameOverScreenLines.push("on the day of his death...");
    stringGameOverScreenLines.push(" ");
    stringGameOverScreenLines.push("The Blacksmith, Wizard, and ");
    stringGameOverScreenLines.push("Merchant would find solace  ");
    stringGameOverScreenLines.push("at the end of the bandits' ");
    stringGameOverScreenLines.push("blades as well.  There would");
    stringGameOverScreenLines.push("be no mercy displayed.  Their");
    stringGameOverScreenLines.push("fate had been sealed when the");
    stringGameOverScreenLines.push("King's final elite guard was ");
    stringGameOverScreenLines.push("freed from his mortal bonds.");

    for (var y = 0; y < stringGameOverScreenLines.length; y++) {
        for (var x = 0; x < stringGameOverScreenLines[y].length; x++) {
            PS.BeadGlyph(x, y, stringGameOverScreenLines[y][x]);
        }
    }
}

function DrawInventory(x, y) {
    for (var itemOffset = 0; itemOffset < hero.inventory.length; itemOffset++) {
        for (var letter = 0; letter <= hero.inventory[itemOffset].Name.length && letter+x < 32; letter++) {
            if (letter == 0) {
                PS.BeadColor(x + letter, y + itemOffset, (hero.inventory[itemOffset].eqSlot > 0 ? PS.COLOR_YELLOW : [64, 64, 64]));
            }
            else {
                PS.BeadGlyph(x + letter, y + itemOffset, hero.inventory[itemOffset].Name[letter - 1]);
            }
        }
    }
}

function DrawEquipment(x, y) {
    var stringWeapon = "Wep:" + hero.eq_Weapon.Name;
    var stringTrinket = "Tri:" + hero.eq_Trinket.Name;
    var stringHelmet = "Hel:" + hero.eq_Helmet.Name;
    var stringChest = "Chs:" + hero.eq_Chest.Name;
    var stringArms = "Arm:" + hero.eq_Arms.Name;
    var stringGloves = "Glv:" + hero.eq_Gloves.Name;
    var stringLeggings = "Leg:" + hero.eq_Leggings.Name;
    var stringBoots = "Bot:" + hero.eq_Boots.Name;

    for (i = 0; i < stringWeapon.length && i < 32; i++) {
        PS.BeadGlyph(x + i, y, stringWeapon[i]);
    }

    y++;
    for (i = 0; i < stringTrinket.length; i++) {
        PS.BeadGlyph(x + i, y, stringTrinket[i]);

    }

    y++;
    for (i = 0; i < stringHelmet.length; i++) {
        PS.BeadGlyph(x + i, y, stringHelmet[i]);
    }

    y++;
    for (i = 0; i < stringChest.length; i++) {
        PS.BeadGlyph(x + i, y, stringChest[i]);
    }

    y++;
    for (i = 0; i < stringArms.length; i++) {
        PS.BeadGlyph(x + i, y, stringArms[i]);
    }

    y++;
    for (i = 0; i < stringGloves.length; i++) {
        PS.BeadGlyph(x + i, y, stringGloves[i]);
    }

    y++;
    for (i = 0; i < stringLeggings.length; i++) {
        PS.BeadGlyph(x + i, y, stringLeggings[i]);
    }

    y++;
    for (i = 0; i < stringBoots.length; i++) {
        PS.BeadGlyph(x + i, y, stringBoots[i]);
    }
}

function Hero(newBaseStrength, newBaseIntelligence, newBaseConstitution, newBaseArmor, newBaseCrit, newGlyph) {
    this.maxHP = 100;
    this.currentHP = 100;
    this.maxMana = 100;
    this.currentMana = 100;
    this.Gold = 0;

    this.baseStrength = newBaseStrength;
    this.Strength;
    this.baseIntelligence = newBaseIntelligence;
    this.Intelligence;
    this.baseConstitution = newBaseConstitution;
    this.Constitution;
    this.baseArmor = newBaseArmor;
    this.Armor;
    this.baseCriticalHitChance = newBaseCrit; // Every 10 points is 1%; So 50 points = 5%, 100 points = 10%, 1000 points = 100%, etc.
    this.CriticalHitChance;
    this.eq_Weapon;
    this.eq_Trinket;
    this.eq_Helmet;
    this.eq_Chest;
    this.eq_Arms;
    this.eq_Gloves;
    this.eq_Leggings;
    this.eq_Boots;
    this.name = "You";
    this.friendly = true;
    this.potentialTarget = 0;
    this.glyph = newGlyph;

    this.worldPositionX = 32;
    this.worldPositionY = 32;

    this.cameraRelativeX = 16;
    this.cameraRelativeY = 11;
    this.blacksmithUpgrades = 0;

    this.spellsLearned = new Array(8);

    this.inventory = new Array(8);
    
    this.CalculateStats = function () {
        this.Strength = this.baseStrength;
        this.Intelligence = this.baseIntelligence;
        this.Constitution = this.baseConstitution;
        this.Armor = this.baseArmor;
        this.CriticalHitChance = this.baseCriticalHitChance;

        // Calculate the character's strength
        this.Strength += this.eq_Weapon.strVal;
        this.Strength += this.eq_Trinket.strVal;
        this.Strength += this.eq_Helmet.strVal;
        this.Strength += this.eq_Chest.strVal;
        this.Strength += this.eq_Arms.strVal;
        this.Strength += this.eq_Gloves.strVal;
        this.Strength += this.eq_Leggings.strVal;
        this.Strength += this.eq_Boots.strVal;

        // Calculate the character's intelligence
        this.Intelligence += this.eq_Weapon.intVal;
        this.Intelligence += this.eq_Trinket.intVal;
        this.Intelligence += this.eq_Helmet.intVal;
        this.Intelligence += this.eq_Chest.intVal;
        this.Intelligence += this.eq_Arms.intVal;
        this.Intelligence += this.eq_Gloves.intVal;
        this.Intelligence += this.eq_Leggings.intVal;
        this.Intelligence += this.eq_Boots.intVal;

        // Calculate the character's constitution
        this.Constitution += this.eq_Weapon.conVal;
        this.Constitution += this.eq_Trinket.conVal;
        this.Constitution += this.eq_Helmet.conVal;
        this.Constitution += this.eq_Chest.conVal;
        this.Constitution += this.eq_Arms.conVal;
        this.Constitution += this.eq_Gloves.conVal;
        this.Constitution += this.eq_Leggings.conVal;
        this.Constitution += this.eq_Boots.conVal;

        // Calculate the character's armor
        this.Armor += this.eq_Weapon.armVal;
        this.Armor += this.eq_Trinket.armVal;
        this.Armor += this.eq_Helmet.armVal;
        this.Armor += this.eq_Chest.armVal;
        this.Armor += this.eq_Arms.armVal;
        this.Armor += this.eq_Gloves.armVal;
        this.Armor += this.eq_Leggings.armVal;
        this.Armor += this.eq_Boots.armVal;

        // Calculate the character's armor
        this.CriticalHitChance += this.eq_Weapon.crtVal;
        this.CriticalHitChance += this.eq_Trinket.crtVal;
        this.CriticalHitChance += this.eq_Helmet.crtVal;
        this.CriticalHitChance += this.eq_Chest.crtVal;
        this.CriticalHitChance += this.eq_Arms.crtVal;
        this.CriticalHitChance += this.eq_Gloves.crtVal;
        this.CriticalHitChance += this.eq_Leggings.crtVal;
        this.CriticalHitChance += this.eq_Boots.crtVal;

        var healthPercentage = this.currentHP / this.maxHP;
        this.maxHP = Math.round(10 * this.Constitution);
        this.currentHP = Math.round(this.maxHP * healthPercentage);

        var manaPercentage = this.currentMana / this.maxMana;
        this.maxMana = Math.round(10 * this.Intelligence);
        this.currentMana = Math.round(this.maxMana * manaPercentage);

        dynamicUINeedsRefreshed = true;
    }

    this.AcquiredItem = function (newItem) {
        for (var i = 0; i < this.inventory.length; i++) {
            if (this.inventory[i].eqSlot == -1) {
                PS.StatusText("Looted an item!");
                this.inventory[i] = newItem;
                break;
            }
        }
    }

    this.UpgradeItems = function () {
        this.eq_Weapon.strVal *= 1.1;
        this.eq_Weapon.intVal *= 1.1;
        this.eq_Weapon.conVal *= 1.1;
        this.eq_Weapon.armVal *= 1.1;
        this.eq_Weapon.crtVal *= 1.1;

        this.eq_Trinket.strVal *= 1.1;
        this.eq_Trinket.intVal *= 1.1;
        this.eq_Trinket.conVal *= 1.1;
        this.eq_Trinket.armVal *= 1.1;
        this.eq_Trinket.crtVal *= 1.1;

        this.eq_Helmet.strVal *= 1.1;
        this.eq_Helmet.intVal *= 1.1;
        this.eq_Helmet.conVal *= 1.1;
        this.eq_Helmet.armVal *= 1.1;
        this.eq_Helmet.crtVal *= 1.1;

        this.eq_Chest.strVal *= 1.1;
        this.eq_Chest.intVal *= 1.1;
        this.eq_Chest.conVal *= 1.1;
        this.eq_Chest.armVal *= 1.1;
        this.eq_Chest.crtVal *= 1.1;

        this.eq_Arms.strVal *= 1.1;
        this.eq_Arms.intVal *= 1.1;
        this.eq_Arms.conVal *= 1.1;
        this.eq_Arms.armVal *= 1.1;
        this.eq_Arms.crtVal *= 1.1;

        this.eq_Gloves.strVal *= 1.1;
        this.eq_Gloves.intVal *= 1.1;
        this.eq_Gloves.conVal *= 1.1;
        this.eq_Gloves.armVal *= 1.1;
        this.eq_Gloves.crtVal *= 1.1;

        this.eq_Leggings.strVal *= 1.1;
        this.eq_Leggings.intVal *= 1.1;
        this.eq_Leggings.conVal *= 1.1;
        this.eq_Leggings.armVal *= 1.1;
        this.eq_Leggings.crtVal *= 1.1;

        this.eq_Boots.strVal *= 1.1;
        this.eq_Boots.intVal *= 1.1;
        this.eq_Boots.conVal *= 1.1;
        this.eq_Boots.armVal *= 1.1;
        this.eq_Boots.crtVal *= 1.1;

        this.blacksmithUpgrades++;

        this.CalculateStats();
        dynamicUINeedsRefreshed = true;
    }

    this.InitializeStats = function () {
        this.maxHP = 100;
        this.currentHP = this.maxHP;
        this.maxMana = 100;
        this.currentMana = this.maxMana;
        this.baseStrength = 10;
        this.baseIntelligence = 10;
        this.baseConstitution = 10;
        this.baseArmor = 2;
        this.eq_Weapon = new Item("Wood Sword", 2, 0, 0, 0, 5, 1, 5);
        this.eq_Trinket = new Item("Wood Totem", 0, 1, 0, 0, 5, 2, 5);
        this.eq_Helmet = new Item("Wood Hat", 0, 0, 1, 2, 0, 3, 5);
        this.eq_Chest = new Item("Wood Tunic", 0, 0, 1, 4, 0, 4, 5);
        this.eq_Arms = new Item("Wood Sleeves", 0, 0, 1, 3, 0, 5, 5);
        this.eq_Gloves = new Item("Wood Gloves", 0, 0, 0, 2, 0, 6, 5);
        this.eq_Leggings = new Item("Wood Greaves", 0, 0, 1, 4, 0, 7, 5);
        this.eq_Boots = new Item("Wood Boots", 0, 0, 0, 2, 0, 8, 5);
        this.blacksmithUpgrades = 0;

        for (var i = 0; i < this.inventory.length; i++) {
            this.inventory[i] = defaultEmptyItem;
        }

        for (var i = 0; i < this.spellsLearned.length; i++) {
            this.spellsLearned[i] = false;
        }
    }

    this.NewPlayerPosition = function (direction) {
        var x, y;
        x = this.cameraRelativeX;
        y = this.cameraRelativeY;

        PS.BeadData(x, y).occupiedBy = 0;

        switch (direction) {
            case 0: // Up
                y--;
                break;
            case 1: // Right
                x++;
                break;
            case 2: // Down
                y++;
                break;
            case 3: // Left
                x--;
                break;
        }

        var destination = PS.BeadData(x, y);

        if (destination.isPassable && (destination.occupiedBy == 0)) {
            //PS.BeadData(this.cameraRelativeX, this.cameraRelativeY).occupiedBy = 0;

            switch (direction) {
                case 0: // Up
                    this.worldPositionY--;
                    break;
                case 1: // Right
                    this.worldPositionX++;
                    break;
                case 2: // Down
                    this.worldPositionY++;
                    break;
                case 3: // Left
                    this.worldPositionX--;
                    break;
            }
            PS.BeadData(x, y).occupiedBy = this;
            worldViewNeedsRefreshed = true;
            aisTurn = true;
        }
    }

    // When I made this function, I realized it would have been much more convenient if I would have 
    // made the equipment into an array instead of naming each slot individually...
    this.EquipItem = function (offset) {
        if (this.inventory[offset].eqSlot > 0) {
            var tempItem;

            switch (this.inventory[offset].eqSlot) {
                case 1: // Weapon
                    tempItem = this.eq_Weapon;
                    this.eq_Weapon = this.inventory[offset];
                    this.inventory[offset] = tempItem;
                    break;
                case 2: // Trinket
                    tempItem = this.eq_Trinket;
                    this.eq_Trinket = this.inventory[offset];
                    this.inventory[offset] = tempItem;
                    break;
                case 3: // Helmet
                    tempItem = this.eq_Helmet;
                    this.eq_Helmet = this.inventory[offset];
                    this.inventory[offset] = tempItem;
                    break;
                case 4: // Chest
                    tempItem = this.eq_Chest;
                    this.eq_Chest = this.inventory[offset];
                    this.inventory[offset] = tempItem;
                    break;
                case 5: // Arms
                    tempItem = this.eq_Arms;
                    this.eq_Arms = this.inventory[offset];
                    this.inventory[offset] = tempItem;
                    break;
                case 6: // Gloves
                    tempItem = this.eq_Gloves;
                    this.eq_Gloves = this.inventory[offset];
                    this.inventory[offset] = tempItem;
                    break;
                case 7: // Leggings
                    tempItem = this.eq_Leggings;
                    this.eq_Leggings = this.inventory[offset];
                    this.inventory[offset] = tempItem;
                    break;
                case 8: // Boots
                    tempItem = this.eq_Boots;
                    this.eq_Boots = this.inventory[offset];
                    this.inventory[offset] = tempItem;
                    break;
            }

            this.CalculateStats();
        }
    }
}

function ArtificialLifeform(Name, friendlyToHero, newBaseStrength, newBaseIntelligence, newBaseArmor, newBaseCritHit, newGlyph, newMaxHP, newGoldValue, worldPosition_X, worldPosition_Y) {
    this.name = Name;
    this.friendly = friendlyToHero;
    this.Strength = newBaseStrength;
    this.Intelligence = newBaseIntelligence;
    this.Armor = newBaseArmor;
    this.CriticalHitChance = newBaseCritHit;
    this.glyph = newGlyph;
    this.maxHP = newMaxHP;
    this.currentHP;
    this.goldValue = newGoldValue;
    this.worldPosition_X = worldPosition_X;
    this.worldPosition_Y = worldPosition_Y;
    this.Loot = new Array();

    this.Initialize = function () {
        this.currentHP = this.maxHP;
        theWorld.worldTerrain[this.worldPosition_X][this.worldPosition_Y].occupiedBy = this;
        worldViewNeedsRefreshed = true;
        this.Loot = defaultEmptyItem;

        if (PS.Random(100) <= 5 && !this.friendly) { // 5% chance for the bandit to carry a random item!
            this.Loot = RandomItem();
        }
    }

    this.ProcessThoughtPattern = function () {
        var heroAllyInRange = false;

        if (!this.friendly) {
            for (var i = 0; i < alliesInMap.length && heroAllyInRange == false; i++) {
                if (Math.abs(this.worldPosition_X - alliesInMap[i].worldPosition_X) <= 1 && Math.abs(this.worldPosition_Y - alliesInMap[i].worldPosition_Y) <= 1) {
                    Attack(this, alliesInMap[i]);
                    heroAllyInRange = true;
                }
            }

            if (!heroAllyInRange) {
                if (Math.abs(this.worldPosition_X - hero.worldPositionX) <= 1 && Math.abs(this.worldPosition_Y - hero.worldPositionY) <= 1) {
                    Attack(this, hero);
                }
                else {
                    this.UnitMove();
                }
            }
        }
        else {
        }

    }

    this.UnitMove = function () {

        var x, y;
        x = this.worldPosition_X;
        y = this.worldPosition_Y;
        
        var direction = PS.Random(4) - 1;
        var failedAttempts = 0;

        for (var moved = false; moved == false && failedAttempts < 4; direction = PS.Random(4)) {

            switch (direction) {
                case 0: // Up
                    y--;
                    break;
                case 1: // Right
                    x++;
                    break;
                case 2: // Down
                    y++;
                    break;
                case 3: // Left
                    x--;
                    break;
            }

            var destination = theWorld.worldTerrain[x][y];

            if (destination.isPassable && (destination.occupiedBy == 0)) {

                theWorld.worldTerrain[this.worldPosition_X][this.worldPosition_Y].occupiedBy = 0;

                moved = true;

                this.worldPosition_X = x;
                this.worldPosition_Y = y;

                theWorld.worldTerrain[this.worldPosition_X][this.worldPosition_Y].occupiedBy = this;
                worldViewNeedsRefreshed = true;
            }
            else {
                failedAttempts++;
                x = this.worldPosition_X;
                y = this.worldPosition_Y;
                theWorld.worldTerrain[x][y].occupiedBy = this;
            }
        }
    }
}

function Terrain(newTerrainType, newTerrainColor, newIsPassable) {
    this.terrainType = newTerrainType;
    this.terrainColor = newTerrainColor;
    this.isPassable = newIsPassable;
    this.occupiedBy = 0;
}

function Item(itemName, strengthValue, intValue, constitutionValue, armorValue, critHitChanceValue, slot, newValue) {
    this.Name = itemName;
    this.strVal = strengthValue;
    this.intVal = intValue;
    this.armVal = armorValue;
    this.conVal = constitutionValue;
    this.crtVal = critHitChanceValue;
    // Slots breakdown:
    // -1 = empty inventory slot flag
    // 0 = unequippable
    // 1 = weapon
    // 2 = trinket
    // 3 = helm
    // 4 = tunic
    // 5 = sleeves
    // 6 = gloves
    // 7 = legs/greaves
    // 8 = boots
    this.eqSlot = slot;
    this.value = newValue;
}

function RandomItem() {
    var randomItem = new Item(" ", 0, 0, 0, 0, 0, 0, 0);
    var itemName;
    var itemPrefix;

    randomItem.eqSlot = PS.Random(8);
    switch (randomItem.eqSlot) {
        case 1:
            randomItem.Name = "Bandit Sword";
            randomItem.strVal = PS.Random(100) % PS.Random(100); // Constrained by a random to make top end bandit weapons rarer
            randomItem.intVal = PS.Random(100) % PS.Random(100);
            randomItem.armVal = PS.Random(5) % PS.Random(5);
            randomItem.conVal = PS.Random(5) % PS.Random(5);
            randomItem.crtVal = PS.Random(150) % PS.Random(150); // Up to 15% crit from a single item
            randomItem.value = PS.Random(200) % PS.Random(200);

            itemPrefix = "Bandit's";
            itemName = "Sword";

            if (randomItem.strVal == 100) {
                itemName = "Halberd";
            }
            else if (randomItem.strVal == 100) {
                itemName = "Staff";
            }
            else if (randomItem.intVal > 75) {
                itemName = "Cane";
            }
            else if (randomItem.strVal > 75) {
                itemName = "Cleaver";
            }
            else if (randomItem.intVal > 50) {
                itemName = "Wand";
            }
            else if (randomItem.intVal > 25) {
                itemName = "Kryss";
            }
            else if (randomItem.strVal > 50) {
                itemName = "Hammer";
            }
            else if (randomItem.strVal > 25) {
                itemName = "Lance";
            }

            if (randomItem.crtVal == 150) {
                itemPrefix = "Serrated";
            }
            else if (randomItem.crtVal > 100) {
                itemPrefix = "Barbed";
            }
            else if (randomItem.crtVal > 50) {
                itemPrefix = "Spiked";
            }
            else if (randomItem.strVal >= randomItem.intVal) {
                itemPrefix = "Warrior's";
            }
            else {
                itemPrefix = "Wizard's";
            }

            randomItem.Name = itemPrefix + " " + itemName;
            break;
        case 2:
            randomItem.Name = "Bandit Trinket";
            randomItem.strVal = PS.Random(10) % PS.Random(10); // Constrained by a random to make top end bandit weapons rarer
            randomItem.intVal = PS.Random(40) % PS.Random(40);
            randomItem.armVal = PS.Random(5) % PS.Random(5);
            randomItem.conVal = PS.Random(5) % PS.Random(5);
            randomItem.crtVal = PS.Random(200) % PS.Random(200); // Up to 15% crit from a single item
            randomItem.value = PS.Random(200) % PS.Random(200);
            break;
        case 3:
            randomItem.Name = "Bandit Helm";
            randomItem.strVal = PS.Random(20) % PS.Random(20); // Constrained by a random to make top end bandit weapons rarer
            randomItem.intVal = PS.Random(100) % PS.Random(100);
            randomItem.armVal = PS.Random(20) % PS.Random(20);
            randomItem.conVal = PS.Random(20) % PS.Random(20);
            randomItem.crtVal = PS.Random(50) % PS.Random(50); // Up to 15% crit from a single item
            randomItem.value = PS.Random(200) % PS.Random(200);
            break;
        case 4:
            randomItem.Name = "Bandit Tunic";
            randomItem.strVal = PS.Random(25) % PS.Random(25); // Constrained by a random to make top end bandit weapons rarer
            randomItem.intVal = PS.Random(15) % PS.Random(15);
            randomItem.armVal = PS.Random(40) % PS.Random(40);
            randomItem.conVal = PS.Random(100) % PS.Random(100);
            randomItem.crtVal = PS.Random(25) % PS.Random(25); // Up to 15% crit from a single item
            randomItem.value = PS.Random(200) % PS.Random(200);
            break;
        case 5:
            randomItem.Name = "Bandit Sleeves";
            randomItem.strVal = PS.Random(100) % PS.Random(100); // Constrained by a random to make top end bandit weapons rarer
            randomItem.intVal = PS.Random(15) % PS.Random(15);
            randomItem.armVal = PS.Random(20) % PS.Random(20);
            randomItem.conVal = PS.Random(15) % PS.Random(15);
            randomItem.crtVal = PS.Random(150) % PS.Random(150); // Up to 15% crit from a single item
            randomItem.value = PS.Random(200) % PS.Random(200);
            break;
        case 6:
            randomItem.Name = "Bandit Gloves";
            randomItem.strVal = PS.Random(65) % PS.Random(65); // Constrained by a random to make top end bandit weapons rarer
            randomItem.intVal = PS.Random(15) % PS.Random(15);
            randomItem.armVal = PS.Random(5) % PS.Random(5);
            randomItem.conVal = PS.Random(5) % PS.Random(5);
            randomItem.crtVal = PS.Random(100) % PS.Random(100); // Up to 15% crit from a single item
            randomItem.value = PS.Random(200) % PS.Random(200);
            break;
        case 7:
            randomItem.Name = "Bandit Greaves";
            randomItem.strVal = PS.Random(15) % PS.Random(15); // Constrained by a random to make top end bandit weapons rarer
            randomItem.intVal = PS.Random(25) % PS.Random(25);
            randomItem.armVal = PS.Random(40) % PS.Random(40);
            randomItem.conVal = PS.Random(65) % PS.Random(65);
            randomItem.crtVal = PS.Random(40) % PS.Random(40); // Up to 15% crit from a single item
            randomItem.value = PS.Random(200) % PS.Random(200);
            break;
        case 8:
            randomItem.Name = "Bandit Boots";
            randomItem.strVal = PS.Random(30) % PS.Random(30); // Constrained by a random to make top end bandit weapons rarer
            randomItem.intVal = PS.Random(50) % PS.Random(50);
            randomItem.armVal = PS.Random(5) % PS.Random(5);
            randomItem.conVal = PS.Random(5) % PS.Random(5);
            randomItem.crtVal = PS.Random(150) % PS.Random(150); // Up to 15% crit from a single item
            randomItem.value = PS.Random(200) % PS.Random(200);
            break;
    }

    return randomItem;
}

function World() {
    this.worldWidth = 64;
    this.worldHeight = 64;
    this.worldTerrain = new Array(this.worldWidth);

    this.Setup = function () {
        for (var i = 0; i < 64; i++) {
            this.worldTerrain[i] = new Array(64);
        }

        // This huge list generates this map...

        for (var x = 0; x < this.worldWidth; x++) {
            for (var y = 0; y < this.worldHeight; y++) {

                this.worldTerrain[x][y] = new Terrain(PS.Random(5), [PS.Random(255), PS.Random(255), PS.Random(255)], true);

                if (x == 0 || y == 0 || x == 63 || y == 63) {
                    this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    // Make the outer perimeter wall
                }
                else if ((x >= 1 && x <= 3) || (x >= 60 && x <= 62)) {
                    if ((y > 0 && y < 30) || (y > 32 && y < 63)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if ((x >= 4 && x <= 6) || (x >= 57 && x <= 59)) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if (x == 7 || x == 56) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else if ((y >= 7 && y <= 17) || (y >= 46 && y <= 56)) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if (x == 8 || x == 55) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else if (y == 7 || y == 17 || y == 46 || y == 56) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if (x == 9 || x == 54) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else if (y == 7 || y == 17 || y == 46 || y == 56 || (y >= 22 && y <= 41)) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if (x == 10 || x == 53) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else if (y == 7 || y == 22 || y == 41 || y == 56) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if (x == 11 || x == 12 || x == 51 || x == 52) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else if (y == 7 || y == 17 || y == 22 || y == 41 || y == 46 || y == 56) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if (x == 13 || x == 50) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else if ((y >= 7 && y <= 17) || (y >= 46 && y <= 56) || ((y >= 22 && y <= 41)) && y != 31) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if ((x >= 14 && x <= 19) || (x >= 43 && x <= 49)) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if (x == 20 || x == 42) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else if (y >= 18 && y <= 42) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if (x == 21 || x == 22 || (x >= 39 && x <= 41)) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else if (y == 18 || y == 42) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if (x == 23 || x == 38) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else if (y == 18 || y == 42 || (y >= 9 && y <= 14)) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if ((x >= 24 && x <= 29) || (x >= 33 && x <= 37)) {
                    if ((y >= 1 && y <= 3) || (y >= 60 && y <= 62)) {
                        this.worldTerrain[x][y] = new Terrain(1, [0, 0, 256], false);
                    }
                    else if (y == 9 || y == 14 || y == 18 || y == 42) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else if (x == 30 || x == 32) {
                    if (y == 9 || y == 14 || y == 18 || y == 42) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
                else {
                    if (y == 9 || y == 18) {
                        this.worldTerrain[x][y] = new Terrain(0, [64, 64, 64], false);
                    }
                    else {
                        this.worldTerrain[x][y] = new Terrain(2, [256, 256, 256], true);
                    }
                }
            }
        }
    }

    this.Draw = function (heroX, heroY) {
        var xDraw = 0;
        for (var x = heroX - 16; x < heroX + 16; x++) {
            var yDraw = 0;
            for (var y = heroY - 11; y < heroY + 12; y++) {

                if (xDraw != 16 || yDraw != 11) {
                    PS.BeadGlyph(xDraw, yDraw, " ");
                }

                if (y >= 64) {
                    PS.BeadData(xDraw, yDraw, new Terrain(1, PS.COLOR_BLACK, false));
                    PS.BeadColor(xDraw, yDraw, PS.BeadData(xDraw, yDraw).terrainColor);
                }
                else if (y < 0) {
                    PS.BeadData(xDraw, yDraw, new Terrain(1, PS.COLOR_BLACK, false));
                    PS.BeadColor(xDraw, yDraw, PS.BeadData(xDraw, yDraw).terrainColor);
                }
                else if (x >= 64) {
                    PS.BeadData(xDraw, yDraw, new Terrain(1, PS.COLOR_BLACK, false));
                    PS.BeadColor(xDraw, yDraw, PS.BeadData(xDraw, yDraw).terrainColor);
                }
                else if (x < 0) {
                    PS.BeadData(xDraw, yDraw, new Terrain(1, PS.COLOR_BLACK, false));
                    PS.BeadColor(xDraw, yDraw, PS.BeadData(xDraw, yDraw).terrainColor);
                }
                else {
                    PS.BeadData(xDraw, yDraw, this.worldTerrain[x][y]);
                    PS.BeadColor(xDraw, yDraw, this.worldTerrain[x][y].terrainColor);

                    if (this.worldTerrain[x][y].occupiedBy != 0) {
                        PS.BeadGlyph(xDraw, yDraw, this.worldTerrain[x][y].occupiedBy.glyph);
                    }
                }

                yDraw++;
            }
            xDraw++;
        }

        worldViewNeedsRefreshed = false;
    }
}